/**
 * <p>File: TestEquationSolver.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: May 2009</p>
 * @author Faton M. Berisha
 * @version 4.01
 */
import numeric.*;

//TODO Separate the examples into separate classes Exm020101...
public class TestEquationSolver {
	private static class TestPolynomial implements Differentiable {
		public double value(double x) {
			return x*x*x + 4*x*x - 10;
		}
		public double derivative(double x) {
			return 3*x*x + 8*x;
		}
	}

	private static class G1 implements Function {
		public double value(double x) {
			return x - x*x*x - 4*x*x + 10;
		}
	}

	private static class G2 implements Function {
		public double value(double x) {
			return Math.sqrt(10.0/x - 4*x);
		}
	}

	private static class G3 implements Function {
		public double value(double x) {
			return Math.sqrt(10 - x*x*x) / 2;
		}
	}

	private static class G4 implements Function {
		public double value(double x) {
			return Math.sqrt(10.0 / (4 + x));
		}
	}

	private static class TestFunction implements Function {
		public double value(double x) {
			return Math.cos(x);
		}
	}

	private static class TestDifferentiable implements Differentiable {
		public double value(double x) {
			return Math.cos(x) - x;
		}

		public double derivative(double x) {
			return -Math.sin(x) - 1;
		}
	}

	public static void main(String[] args) {
		EquationSolver solver = new EquationSolver();
		solver.setTolerance(1E-5);
		solver.setNumIterations(20);
		EquationDemoView.setNumSignificantDigits(5);

		// Example 2.1.1
/*		TestPolynomial pol = new TestPolynomial();
		new EquationDemoView(solver);
		try {
			solver.bisection(pol, 1, 2);
		} catch (ArithmeticException e) { }
*/		


		// Example 2.2.3
/*		TestPolynomial pol = new TestPolynomial();
		new EquationDemoView("g1", solver);
		try {
			solver.fixedPoint(new G1(), 1.5);
		} catch (ArithmeticException e) { }
		new EquationDemoView("g2", solver);
		try {
			solver.fixedPoint(new G2(), 1.5);
		} catch (ArithmeticException e) { }
		new EquationDemoView("g3", solver);
		try {
			solver.fixedPoint(new G3(), 1.5);
		} catch (ArithmeticException e) { }
		new EquationDemoView("g4", solver);
		try {
			solver.fixedPoint(new G4(), 1.5);
		} catch (ArithmeticException e) { }
		new EquationDemoView("g5", solver);
		try {
			solver.newton(pol, 1.5);
		} catch (ArithmeticException e) { }
*/		

		//Example 2.3.1-2
   		TestFunction g = new TestFunction();
		TestDifferentiable f = new TestDifferentiable();
		new EquationDemoView("Metoda e biseksionit", solver);
		try {
			solver.bisection(f, 0, Math.PI / 2);
		} catch (ArithmeticException e) { }
		new EquationDemoView("Iterimi me pik fikse", solver);
		try {
			solver.fixedPoint(g, Math.PI / 4);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}
		new EquationDemoView("Metoda e Newton-it", solver);
		try {
			solver.newton(f, Math.PI / 4);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}
		new EquationDemoView("Metoda e sekants", solver);
		try {
			solver.secant(f, 0.5, Math.PI / 4);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

	}
}
